@@ -24,9 +24,15 @@ from dash.exceptions import PreventUpdate
 import plotly.graph_objs as go
 
 from ..dashboard_methods import *
+from .. import to_html
 
 
 class ClassifierRandomIndexComponent(ExplainerComponent):
+
+    _state_props = dict(
+        index=('random-index-clas-index-', 'value')
+    )
+
     def __init__(self, explainer, title="Select Random Index", name=None,
                         subtitle="Select from list or pick at random",
                         hide_title=False, hide_subtitle=False, 
@@ -202,6 +208,14 @@ class ClassifierRandomIndexComponent(ExplainerComponent):
             ]),
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        
+        html = to_html.card(f"Selected index: <b>{self.explainer.get_index(args['index'])}</b>", title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('random-index-clas-index-'+self.name, 'value'),
@@ -248,6 +262,12 @@ class ClassifierRandomIndexComponent(ExplainerComponent):
 
 
 class ClassifierPredictionSummaryComponent(ExplainerComponent):
+
+    _state_props = dict(
+        index=('clas-prediction-index-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
+
     def __init__(self, explainer, title="Prediction", name=None,
                     hide_index=False, hide_title=False,  hide_subtitle=False,
                     hide_table=False, hide_piechart=False, 
@@ -342,6 +362,49 @@ class ClassifierPredictionSummaryComponent(ExplainerComponent):
             ])
         ])
 
+    def _format_preds_df(self, preds_df):              
+        preds_df.probability = np.round(100*preds_df.probability.values, self.round).astype(str)
+        preds_df.probability = preds_df.probability + ' %'
+        if 'logodds' in preds_df.columns:
+            preds_df.logodds = np.round(preds_df.logodds.values, self.round).astype(str)
+        
+        if self.explainer.model_output != 'logodds':
+            preds_df = preds_df[['label', 'probability']]
+        return preds_df
+
+    def get_state_tuples(self):
+        _state_tuples = super().get_state_tuples()
+        if self.feature_input_component is not None:
+            _state_tuples.extend(self.feature_input_component.get_state_tuples())
+        return sorted(list(set(_state_tuples)))
+
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        if self.feature_input_component is None:
+            if args['index'] is not None:
+                fig = self.explainer.plot_prediction_result(args['index'], showlegend=False)
+                preds_df = self.explainer.prediction_result_df(args['index'], round=self.round, logodds=True)  
+                preds_df = self._format_preds_df(preds_df)
+                html = to_html.row(to_html.table_from_df(preds_df), to_html.fig(fig)) 
+            else:
+                html = "no index selected"
+        else:
+            inputs = {k:v for k,v in self.feature_input_component.get_state_args(state_dict).items() if k != 'index'}
+            inputs = list(inputs.values())
+            if len(inputs) == len(self.feature_input_component._input_features) and not any([i is None for i in inputs]):
+                X_row = self.explainer.get_row_from_input(inputs, ranked_by_shap=True)
+                fig = self.explainer.plot_prediction_result(X_row=X_row, showlegend=False)
+                preds_df = self.explainer.prediction_result_df(X_row=X_row, round=self.round, logodds=True)  
+                preds_df = self._format_preds_df(preds_df)
+                html = to_html.row(to_html.table_from_df(preds_df), to_html.fig(fig)) 
+            else:
+                html = f"<div>input data incorrect<div>"
+
+        html = to_html.card(html, title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         if self.feature_input_component is None:
             @app.callback(
@@ -353,16 +416,8 @@ class ClassifierPredictionSummaryComponent(ExplainerComponent):
                 if index is None or not self.explainer.index_exists(index):
                     raise PreventUpdate
                 fig = self.explainer.plot_prediction_result(index, showlegend=False)
-
-                preds_df = self.explainer.prediction_result_df(index, round=self.round, logodds=True)                
-                preds_df.probability = np.round(100*preds_df.probability.values, self.round).astype(str)
-                preds_df.probability = preds_df.probability + ' %'
-                if 'logodds' in preds_df.columns:
-                    preds_df.logodds = np.round(preds_df.logodds.values, self.round).astype(str)
-                
-                if self.explainer.model_output != 'logodds':
-                    preds_df = preds_df[['label', 'probability']]
-                    
+                preds_df = self.explainer.prediction_result_df(index, round=self.round, logodds=True)  
+                preds_df = self._format_preds_df(preds_df)
                 preds_table = dbc.Table.from_dataframe(preds_df, 
                                     striped=False, bordered=False, hover=False)  
                 return preds_table, fig
@@ -378,20 +433,22 @@ class ClassifierPredictionSummaryComponent(ExplainerComponent):
                 fig = self.explainer.plot_prediction_result(X_row=X_row, showlegend=False)
 
                 preds_df = self.explainer.prediction_result_df(X_row=X_row, round=self.round, logodds=True)                
-                preds_df.probability = np.round(100*preds_df.probability.values, self.round).astype(str)
-                preds_df.probability = preds_df.probability + ' %'
-                if 'logodds' in preds_df.columns:
-                    preds_df.logodds = np.round(preds_df.logodds.values, self.round).astype(str)
-                
-                if self.explainer.model_output!='logodds':
-                    preds_df = preds_df[['label', 'probability']]
-                    
+                preds_df = self._format_preds_df(preds_df)
                 preds_table = dbc.Table.from_dataframe(preds_df, 
                                     striped=False, bordered=False, hover=False)  
                 return preds_table, fig
 
 
 class PrecisionComponent(ExplainerComponent):
+
+    _state_props = dict(
+        bin_size=('precision-binsize-', 'value'),
+        quantiles=('precision-quantiles-', 'value'),
+        quantiles_or_binsize=('precision-binsize-or-quantiles-', 'value'),
+        cutoff=('precision-cutoff-', 'value'),
+        multiclass=('precision-multiclass-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Precision Plot", name=None,
                     subtitle="Does fraction positive increase with predicted probability?",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -576,6 +633,22 @@ class PrecisionComponent(ExplainerComponent):
             ]), hide=self.hide_footer)   
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        if args['quantiles_or_binsize'] == 'bin_size':
+            fig = self.explainer.plot_precision(
+                    bin_size=args['bin_size'], cutoff=args['cutoff'], 
+                    multiclass=bool(args['multiclass']), pos_label=args['pos_label'])
+        elif args['quantiles_or_binsize'] == 'quantiles':
+            fig = self.explainer.plot_precision(
+                quantiles=args['quantiles'], cutoff=args['cutoff'], 
+                multiclass=bool(args['multiclass']), pos_label=args['pos_label'])
+
+        html = to_html.card(to_html.fig(fig), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             [Output('precision-bin-size-div-'+self.name, 'style'),
@@ -623,12 +696,20 @@ class PrecisionComponent(ExplainerComponent):
 
 
 class ConfusionMatrixComponent(ExplainerComponent):
+    _state_props = dict(
+        cutoff=('confusionmatrix-cutoff-', 'value'),
+        percentage=('confusionmatrix-percentage-', 'value'),
+        normalize=('confusionmatrix-normalize-', 'value'),
+        binary=('confusionmatrix-binary-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )    
     def __init__(self, explainer, title="Confusion Matrix", name=None,
                     subtitle="How many false positives and false negatives?",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
                     hide_cutoff=False, hide_percentage=False, hide_binary=False,
-                    hide_selector=False, hide_popout=False, pos_label=None,
-                    cutoff=0.5, percentage=True, binary=True, description=None,
+                    hide_selector=False, hide_popout=False, hide_normalize=False,
+                    normalize='all', pos_label=None, cutoff=0.5, 
+                    percentage=True, binary=True, description=None,
                     **kwargs):
         """Display confusion matrix component
 
@@ -655,6 +736,8 @@ class ConfusionMatrixComponent(ExplainerComponent):
             binary (bool, optional): Show binary instead of multiclass confusion matrix. Defaults to True.
             description (str, optional): Tooltip to display when hover over
                 component title. When None default text is shown. 
+            normalize (str[‘true’, ‘pred’, ‘all’]): normalizes confusion matrix over the true (rows), predicted (columns) conditions or all the population.
+                Defaults to all
         """
         super().__init__(explainer, title, name)
 
@@ -736,6 +819,24 @@ class ConfusionMatrixComponent(ExplainerComponent):
                             ),
                         ]),
                     ]), hide=self.hide_percentage),
+                make_hideable(
+                    html.Div([
+                        dbc.FormGroup([
+                            dbc.Label("Normalisation:"),
+                            dbc.Tooltip("Normalize the percentages in the confusion matrix over the true observations, predicted values or overall",
+                                    target='confusionmatrix-normalize-'+self.name),
+                            dbc.RadioItems(
+                                options=[
+                                    {"label":  "Overall", "value": 'all'},
+                                    {"label":  "Observed", "value": 'observed'},
+                                    {"label":  "Predicted", "value": 'pred'}
+                                    ],
+                                value=self.normalize,
+                                id='confusionmatrix-normalize-'+self.name,
+                                inline=True,
+                            ),
+                        ]),
+                    ]), hide=self.hide_normalize),                
                 make_hideable(
                     html.Div([
                         dbc.FormGroup([
@@ -755,22 +856,43 @@ class ConfusionMatrixComponent(ExplainerComponent):
                     ]), hide=self.hide_binary),
             ]), hide=self.hide_footer)
         ])
+    
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        args['binary'] = bool(args['binary'])
+        args['percentage'] = bool(args['percentage'])
+        fig = self.explainer.plot_confusion_matrix(cutoff=args['cutoff'], 
+                    percentage=args['percentage'], binary=args['binary'], 
+                    normalize=args['normalize'], pos_label=args['pos_label'])
+        
+        html = to_html.card(to_html.fig(fig), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
 
     def component_callbacks(self, app):
         @app.callback(
              Output('confusionmatrix-graph-'+self.name, 'figure'),
             [Input('confusionmatrix-cutoff-'+self.name, 'value'),
              Input('confusionmatrix-percentage-'+self.name, 'value'),
+             Input('confusionmatrix-normalize-'+self.name, 'value')],
              Input('confusionmatrix-binary-'+self.name, 'value'),
-             Input('pos-label-'+self.name, 'value')],
+             Input('pos-label-'+self.name, 'value'),
         )
-        def update_confusionmatrix_graph(cutoff, normalized, binary, pos_label):
+        def update_confusionmatrix_graph(cutoff, percentage, normalize, binary, pos_label):
             return self.explainer.plot_confusion_matrix(
-                        cutoff=cutoff, normalized=bool(normalized), 
+                        cutoff=cutoff, percentage=bool(percentage), normalize=normalize, 
                         binary=bool(binary), pos_label=pos_label)
 
 
 class LiftCurveComponent(ExplainerComponent):
+
+    _state_props = dict(
+        cutoff=('liftcurve-cutoff-', 'value'),
+        percentage=('liftcurve-percentage-', 'value'),
+        wizard=('liftcurve-wizard-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Lift Curve", name=None,
                     subtitle="Performance how much better than random?",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -899,6 +1021,18 @@ class LiftCurveComponent(ExplainerComponent):
             ]), hide=self.hide_footer)
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        fig = self.explainer.plot_lift_curve(cutoff=args['cutoff'], 
+                percentage=bool(args['percentage']), pos_label=args['pos_label'], 
+                add_wizard=bool(args['wizard']))
+
+        html = to_html.card(fig.to_html(include_plotlyjs='cdn', full_html=False), 
+                                title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('liftcurve-graph-'+self.name, 'figure'),
@@ -914,6 +1048,10 @@ class LiftCurveComponent(ExplainerComponent):
 
 
 class CumulativePrecisionComponent(ExplainerComponent):
+    _state_props = dict(
+        percentile=('cumulative-precision-percentile-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Cumulative Precision", name=None,
                     subtitle="Expected distribution for highest scores",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -1025,6 +1163,15 @@ class CumulativePrecisionComponent(ExplainerComponent):
             ]), hide=self.hide_footer)   
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        fig = self.explainer.plot_cumulative_precision(percentile=args['percentile'], pos_label=args['pos_label'])
+
+        html = to_html.card(to_html.fig(fig), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('cumulative-precision-graph-'+self.name, 'figure'),
@@ -1044,6 +1191,11 @@ class CumulativePrecisionComponent(ExplainerComponent):
 
 
 class ClassificationComponent(ExplainerComponent):
+    _state_props = dict(
+        cutoff=('classification-cutoff-', 'value'),
+        percentage=('classification-percentage-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Classification Plot", name=None,
                     subtitle="Distribution of labels above and below cutoff",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -1153,6 +1305,16 @@ class ClassificationComponent(ExplainerComponent):
             ]), hide=self.hide_footer)
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        fig = self.explainer.plot_classification(
+                    cutoff=args['cutoff'], percentage=bool(args['percentage']), pos_label=args['pos_label'])
+
+        html = to_html.card(to_html.fig(fig), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('classification-graph-'+self.name, 'figure'),
@@ -1166,6 +1328,10 @@ class ClassificationComponent(ExplainerComponent):
 
 
 class RocAucComponent(ExplainerComponent):
+    _state_props = dict(
+        cutoff=('rocauc-cutoff-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="ROC AUC Plot", name=None, 
                     subtitle="Trade-off between False positives and false negatives",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -1250,6 +1416,15 @@ class RocAucComponent(ExplainerComponent):
             ]), hide=self.hide_footer)
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        fig = self.explainer.plot_roc_auc(**args)
+        
+        html = to_html.card(fig.to_html(include_plotlyjs='cdn', full_html=False), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('rocauc-graph-'+self.name, 'figure'),
@@ -1261,6 +1436,10 @@ class RocAucComponent(ExplainerComponent):
 
 
 class PrAucComponent(ExplainerComponent):
+    _state_props = dict(
+        cutoff=('prauc-cutoff-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="PR AUC Plot", name=None,
                     subtitle="Trade-off between Precision and Recall",
                     hide_title=False, hide_subtitle=False, hide_footer=False,
@@ -1348,6 +1527,15 @@ class PrAucComponent(ExplainerComponent):
             ]), hide=self.hide_footer)
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        fig = self.explainer.plot_pr_auc(cutoff=args['cutoff'], pos_label=args['pos_label'])
+
+        html = to_html.card(to_html.fig(fig), title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             Output('prauc-graph-'+self.name, 'figure'),
@@ -1358,6 +1546,10 @@ class PrAucComponent(ExplainerComponent):
             return self.explainer.plot_pr_auc(cutoff=cutoff, pos_label=pos_label)
 
 class ClassifierModelSummaryComponent(ExplainerComponent):
+    _state_props = dict(
+        cutoff=('clas-model-summary-cutoff-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Model performance metrics", name=None,
                     hide_title=False, hide_subtitle=False, hide_footer=False,
                     hide_cutoff=False, hide_selector=False,
@@ -1435,7 +1627,25 @@ class ClassifierModelSummaryComponent(ExplainerComponent):
                     ], style={'margin-bottom': 25}), hide=self.hide_cutoff), 
             ]), hide=self.hide_footer)
         ])
+
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        metrics_df = self._get_metrics_df(args['cutoff'], args['pos_label'])
+        html = to_html.table_from_df(metrics_df)
+        html = to_html.card(html, title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
     
+    def _get_metrics_df(self, cutoff, pos_label):
+        metrics_df = (pd.DataFrame(
+                                self.explainer.metrics(cutoff=cutoff, pos_label=pos_label, 
+                                                        show_metrics=self.show_metrics), 
+                                index=["Score"])
+                              .T.rename_axis(index="metric").reset_index()
+                              .round(self.round))
+        return metrics_df
+
     def component_callbacks(self, app):
         @app.callback(
             Output('clas-model-summary-div-'+self.name, 'children'),
@@ -1443,18 +1653,11 @@ class ClassifierModelSummaryComponent(ExplainerComponent):
              Input('pos-label-'+self.name, 'value')],
         )
         def update_classifier_summary(cutoff, pos_label):
+            metrics_df = self._get_metrics_df(cutoff, pos_label)
+            metrics_table = dbc.Table.from_dataframe(metrics_df, striped=False, bordered=False, hover=False)
             metrics_dict = self.explainer.metrics_descriptions(cutoff, pos_label)
-            metrics_df = (pd.DataFrame(
-                                self.explainer.metrics(cutoff=cutoff, pos_label=pos_label, 
-                                                        show_metrics=self.show_metrics), 
-                                index=["Score"])
-                              .T.rename_axis(index="metric").reset_index()
-                              .round(self.round))
-            metrics_table = dbc.Table.from_dataframe(metrics_df, striped=False, bordered=False, hover=False)        
-            metrics_table, tooltips = get_dbc_tooltips(metrics_table, 
-                                                   metrics_dict, 
-                                                   "clas-model-summary-div-hover", 
-                                                   self.name)
+            metrics_table, tooltips = get_dbc_tooltips(metrics_table, metrics_dict, 
+                                                   "clas-model-summary-div-hover", self.name)
             return html.Div([
                 metrics_table,
                 *tooltips
