@@ -22,11 +22,12 @@ from .overview_components import *
 from .connectors import *
 from .shap_components import *
 from .decisiontree_components import *
+from .. import to_html
 
 
 class ImportancesComposite(ExplainerComponent):
     def __init__(self, explainer, title="Feature Importances", name=None,
-                    hide_importances=False,
+                    hide_title=False, hide_importances=False, hide_descriptions=False,
                     hide_selector=True, **kwargs):
         """Overview tab of feature importances
 
@@ -40,7 +41,9 @@ class ImportancesComposite(ExplainerComponent):
             name (str, optional): unique name to add to Component elements. 
                         If None then random uuid is generated to make sure 
                         it's unique. Defaults to None.
+            hide_title (bool, optional): hide the title
             hide_importances (bool, optional): hide the ImportancesComponent
+            hide_descriptions (bool, optional): hide the FeatureDescriptionsComponent
             hide_selector (bool, optional): hide the post label selector. 
                 Defaults to True.
         """
@@ -48,17 +51,42 @@ class ImportancesComposite(ExplainerComponent):
 
         self.importances = ImportancesComponent(
                 explainer, name=self.name+"0", hide_selector=hide_selector, **kwargs)
+        self.feature_descriptions = FeatureDescriptionsComponent(explainer, **kwargs)
+        
+        if not self.explainer.descriptions:
+            self.hide_descriptions=True
 
     def layout(self):
         return html.Div([
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                     html.H2(self.title)]), hide=self.hide_title),
+            ]),
             dbc.Row([
                 make_hideable(
                     dbc.Col([
                         self.importances.layout(),
                     ]), hide=self.hide_importances),
+            ], style=dict(margin=25)),
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                        self.feature_descriptions.layout(),
+                    ]), hide=self.hide_descriptions),
             ], style=dict(margin=25))
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.hide(to_html.title(self.title), hide=self.hide_title)
+        html += to_html.card_rows(
+            [to_html.hide(self.importances.to_html(state_dict, add_header=False), self.hide_importances)],
+            [to_html.hide(self.feature_descriptions.to_html(state_dict, add_header=False), self.hide_descriptions)],
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 
 class ClassifierModelStatsComposite(ExplainerComponent):
     def __init__(self, explainer, title="Classification Stats", name=None,
@@ -158,6 +186,22 @@ class ClassifierModelStatsComposite(ExplainerComponent):
             ], style=dict(marginBottom=25)),
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.hide(to_html.title(self.title), hide=self.hide_title)
+        html += to_html.card_rows(
+            [to_html.hide(self.summary.to_html(state_dict, add_header=False), hide=self.hide_modelsummary),
+             to_html.hide(self.confusionmatrix.to_html(state_dict, add_header=False), hide=self.hide_confusionmatrix)],
+            [to_html.hide(self.precision.to_html(state_dict, add_header=False), hide=self.hide_precision), 
+             to_html.hide(self.classification.to_html(state_dict, add_header=False), hide=self.hide_classification)],
+            [to_html.hide(self.rocauc.to_html(state_dict, add_header=False), hide=self.hide_rocauc),
+             to_html.hide(self.prauc.to_html(state_dict, add_header=False), hide=self.hide_prauc)],
+            [to_html.hide(self.liftcurve.to_html(state_dict, add_header=False), hide=self.hide_liftcurve),
+             to_html.hide(self.cumulative_precision.to_html(state_dict, add_header=False), hide=self.hide_cumprecision)]
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 
 class RegressionModelStatsComposite(ExplainerComponent):
     def __init__(self, explainer, title="Regression Stats", name=None,
@@ -223,6 +267,18 @@ class RegressionModelStatsComposite(ExplainerComponent):
             ], style=dict(margin=25))
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.hide(to_html.title(self.title), hide=self.hide_title)
+        html += to_html.card_rows(
+            [to_html.hide(self.modelsummary.to_html(state_dict, add_header=False), hide=self.hide_modelsummary),
+             to_html.hide(self.preds_vs_actual.to_html(state_dict, add_header=False), hide=self.hide_predsvsactual)],
+            [to_html.hide(self.residuals.to_html(state_dict, add_header=False), hide=self.hide_residuals),
+             to_html.hide(self.reg_vs_col.to_html(state_dict, add_header=False), hide=self.hide_regvscol)],
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 
 class IndividualPredictionsComposite(ExplainerComponent):
     def __init__(self, explainer, title="Individual Predictions", name=None,
@@ -303,6 +359,19 @@ class IndividualPredictionsComposite(ExplainerComponent):
                 ])
         ], fluid=True)
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.title(self.title)
+        html += to_html.card_rows(
+            [to_html.hide(self.index.to_html(state_dict, add_header=False), self.hide_predindexselector), 
+             to_html.hide(self.summary.to_html(state_dict, add_header=False), self.hide_predictionsummary)],
+            [to_html.hide(self.contributions.to_html(state_dict, add_header=False), self.hide_contributiongraph),
+             to_html.hide(self.pdp.to_html(state_dict, add_header=False), self.hide_pdp)],
+            [to_html.hide(self.contributions_list.to_html(state_dict, add_header=False), self.hide_contributiontable)]
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 
 class WhatIfComposite(ExplainerComponent):
     def __init__(self, explainer, title="What if...", name=None,
@@ -408,6 +477,21 @@ class WhatIfComposite(ExplainerComponent):
                 ])
         ], fluid=True)
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.title(self.title)
+        html += to_html.card_rows(
+            [to_html.hide(self.index.to_html(state_dict, add_header=False), self.hide_whatifindexselector), 
+             to_html.hide(self.prediction.to_html(state_dict, add_header=False), self.hide_whatifprediction)],
+            [to_html.hide(self.input.to_html(state_dict, add_header=False), self.hide_inputeditor)],
+            [to_html.hide(self.contribgraph.to_html(state_dict, add_header=False), self.hide_whatifcontributiongraph),
+            to_html.hide(self.pdp.to_html(state_dict, add_header=False), self.hide_whatifpdp)],
+            [to_html.hide(self.contribtable.to_html(state_dict, add_header=False), self.hide_whatifcontributiontable)]
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
+    
 
 class ShapDependenceComposite(ExplainerComponent):
     def __init__(self, explainer, title='Feature Dependence', name=None,
@@ -448,6 +532,16 @@ class ShapDependenceComposite(ExplainerComponent):
             ], style=dict(marginTop=25)),
         ], fluid=True)
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.title(self.title)
+        html += to_html.card_rows(
+            [to_html.hide(self.shap_summary.to_html(state_dict, add_header=False), self.hide_shapsummary), 
+             to_html.hide(self.shap_dependence.to_html(state_dict, add_header=False), self.hide_shapdependence)],
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 class ShapInteractionsComposite(ExplainerComponent):
     def __init__(self, explainer, title='Feature Interactions', name=None,
                     hide_selector=True,
@@ -484,6 +578,16 @@ class ShapInteractionsComposite(ExplainerComponent):
                 ], style=dict(marginTop=25))
         ], fluid=True)
 
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.title(self.title)
+        html += to_html.card_rows(
+            [to_html.hide(self.interaction_summary.to_html(state_dict, add_header=False), self.hide_interactionsummary), 
+             to_html.hide(self.interaction_dependence.to_html(state_dict, add_header=False), self.hide_interactiondependence)],
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
 
 class DecisionTreesComposite(ExplainerComponent):
     def __init__(self, explainer, title="Decision Trees", name=None,
@@ -538,33 +642,7 @@ class DecisionTreesComposite(ExplainerComponent):
             [self.decisionpath_table, self.decisionpath_graph])
         
     def layout(self):
-        if isinstance(self.explainer, XGBExplainer):
-            return html.Div([
-                dbc.Row([
-                    make_hideable(
-                        dbc.Col([
-                            self.index.layout(), 
-                        ]), hide=self.hide_treeindexselector),
-                ], style=dict(margin=25)),
-                dbc.Row([
-                    make_hideable(
-                        dbc.Col([
-                            self.trees.layout(), 
-                        ], md=8), hide=self.hide_treesgraph),
-                    make_hideable(
-                        dbc.Col([
-                            self.decisionpath_table.layout(), 
-                        ], md=4), hide=self.hide_treepathtable),
-                ], style=dict(margin=25)),
-                dbc.Row([
-                    make_hideable(
-                        dbc.Col([
-                            self.decisionpath_graph.layout()
-                        ]), hide=self.hide_treepathgraph),
-                ], style=dict(margin=25)),
-            ])
-        elif isinstance(self.explainer, RandomForestExplainer):
-            return html.Div([
+        return html.Div([
                 dbc.Row([
                     make_hideable(
                         dbc.Col([
@@ -590,13 +668,22 @@ class DecisionTreesComposite(ExplainerComponent):
                         ]), hide=self.hide_treepathgraph),
                 ], style=dict(margin=25)),
             ])
-        else:
-            raise ValueError("explainer is neither a RandomForestExplainer nor an XGBExplainer! "
-                            "Pass decision_trees=False to disable the decision tree tab.")
+    
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.title(self.title)
+        html += to_html.card_rows(
+            [to_html.hide(self.index.to_html(state_dict, add_header=False), self.hide_treeindexselector)],
+            [to_html.hide(self.trees.to_html(state_dict, add_header=False), self.hide_treesgraph)],
+            [to_html.hide(self.decisionpath_table.to_html(state_dict, add_header=False), self.hide_treepathtable)],
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
+        
 
 class SimplifiedClassifierComposite(ExplainerComponent):
     def __init__(self, explainer, title="Simple Classifier Explainer", name=None,
-                 classifier_custom_component='roc_auc', 
+                 hide_title=False, classifier_custom_component='roc_auc', 
                  hide_confusionmatrix=False, hide_classifier_custom_component=False,
                  hide_shapsummary=False, hide_shapdependence=False,
                  hide_predindexselector=False, hide_predictionsummary=False,
@@ -618,6 +705,7 @@ class SimplifiedClassifierComposite(ExplainerComponent):
             name (str, optional): unique name to add to Component elements. 
                         If None then random uuid is generated to make sure 
                         it's unique. Defaults to None.
+            hide_title (bool, optional): hide the title. Defaults to False.
             classifier_custom_component (str, optional): custom classifier quality indicator 
                     supported by the ClassifierExplainer object. Valid values are: 
                     'roc_auc', 'metrics', pr_auc', 'precision_graph', 'lift_curve', 
@@ -632,10 +720,10 @@ class SimplifiedClassifierComposite(ExplainerComponent):
                 or RegressionPredictionSummaryComponent
             hide_contributiongraph (bool, optional): hide ShapContributionsGraphComponent
         """
-        super().__init__(explainer, title, name)
+        super().__init__(explainer, title=title, name=name)
         
         self.confusionmatrix = ConfusionMatrixComponent(explainer, 
-                                    **update_params(kwargs, hide_percentage=True, hide_selector=True))
+                                    **update_params(kwargs, hide_percentage=True, hide_selector=True, hide_normalize=True))
 
         # select custom classifier report metric
         if classifier_custom_component == 'metrics':
@@ -667,7 +755,7 @@ class SimplifiedClassifierComposite(ExplainerComponent):
             explainer, **update_params(kwargs, title="Shap Feature Importances", 
                                        hide_index=True, hide_selector=True, depth=None, hide_depth=True))
         self.shap_dependence = ShapDependenceComponent( 
-            explainer, **update_params(kwargs, hide_selector=True, hide_index=True))
+            explainer, **update_params(kwargs, hide_selector=True, hide_index=True, color_col="no_color_col"))
 
         # SHAP contribution, along with prediction summary
         self.index = ClassifierRandomIndexComponent(explainer, 
@@ -682,39 +770,71 @@ class SimplifiedClassifierComposite(ExplainerComponent):
         self.index_connector = IndexConnector(self.index, [self.summary, self.contributions])
 
     def layout(self):
-        return html.Div([
-            html.H1(self.title),
-            html.H2("Model performance"),
-            dbc.CardDeck([
-                make_hideable(self.confusionmatrix.layout(),
-                              hide=self.hide_confusionmatrix),
-                make_hideable(self.classifier_custom_component.layout(),
-                              hide=self.hide_classifier_custom_component),
-            ], style=dict(marginTop=25, marginBottom=25)),
-            html.H3("SHAP values"),
-            dbc.CardDeck([
-                make_hideable(self.shap_summary.layout(),
-                              hide=self.hide_shapsummary),
-                make_hideable(self.shap_dependence.layout(),
-                              hide=self.hide_shapdependence),
-            ], style=dict(marginTop=25, marginBottom=25)),
-            html.H2("Individual predictions"),
-            dbc.CardDeck([
-                make_hideable(self.index.layout(),
-                              hide=self.hide_predindexselector),
-                make_hideable(self.summary.layout(),
-                              hide=self.hide_predictionsummary),
-            ], style=dict(marginBottom=25, marginTop=25)),
-            dbc.CardDeck([
-                make_hideable(self.contributions.layout(),
-                              hide=self.hide_contributiongraph),
-            ], style=dict(marginBottom=25, marginTop=25))
-        ])
+        return dbc.Container([
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                        html.H1(self.title, id='simple-classifier-composite-title')
+                    ]), hide=self.hide_title),
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H2("Model performance"),
+                    dbc.CardDeck([
+                        make_hideable(self.confusionmatrix.layout(),
+                                    hide=self.hide_confusionmatrix),
+                        make_hideable(self.classifier_custom_component.layout(),
+                                    hide=self.hide_classifier_custom_component),
+                    ], style=dict(marginTop=25, marginBottom=25)),
+                ])
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H3("SHAP values"),
+                    dbc.CardDeck([
+                        make_hideable(self.shap_summary.layout(),
+                                    hide=self.hide_shapsummary),
+                        make_hideable(self.shap_dependence.layout(),
+                                    hide=self.hide_shapdependence),
+                    ], style=dict(marginTop=25, marginBottom=25)),
+                ])
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H2("Individual predictions"),
+                    dbc.CardDeck([
+                        make_hideable(self.index.layout(),
+                                    hide=self.hide_predindexselector),
+                        make_hideable(self.summary.layout(),
+                                    hide=self.hide_predictionsummary),
+                    ], style=dict(marginBottom=25, marginTop=25)),
+                    dbc.CardDeck([
+                        make_hideable(self.contributions.layout(),
+                                    hide=self.hide_contributiongraph),
+                    ], style=dict(marginBottom=25, marginTop=25))
+                ])
+            ]),  
+        ], fluid=False)
+
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.hide(to_html.title(self.title), hide=self.hide_title)
+        html += to_html.card_rows(
+            [to_html.hide(self.confusionmatrix.to_html(state_dict, add_header=False), hide=self.hide_confusionmatrix),
+             to_html.hide(self.classifier_custom_component.to_html(state_dict, add_header=False), hide=self.hide_classifier_custom_component)],
+            [to_html.hide(self.shap_summary.to_html(state_dict, add_header=False), hide=self.hide_shapsummary),
+             to_html.hide(self.shap_dependence.to_html(state_dict, add_header=False), hide=self.hide_shapdependence)],
+            [to_html.hide(self.index.to_html(state_dict, add_header=False), hide=self.hide_predindexselector),
+             to_html.hide(self.summary.to_html(state_dict, add_header=False), hide=self.hide_predictionsummary)],
+            [to_html.hide(self.contributions.to_html(state_dict, add_header=False), hide=self.hide_contributiongraph)]
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
 
 
 class SimplifiedRegressionComposite(ExplainerComponent):
     def __init__(self, explainer, title="Simple Regression Explainer", name=None,
-                 regression_custom_component='vs_col', 
+                 hide_title=False, regression_custom_component='vs_col', 
                  hide_goodness_of_fit=False, hide_regression_custom_component=False,
                  hide_shapsummary=False, hide_shapdependence=False,
                  hide_predindexselector=False, hide_predictionsummary=False,
@@ -735,6 +855,7 @@ class SimplifiedRegressionComposite(ExplainerComponent):
             name (str, optional): unique name to add to Component elements. 
                         If None then random uuid is generated to make sure 
                         it's unique. Defaults to None.
+            hide_title (bool, optional): hide the title. Defaults to False.
             regression_custom_component (str, optional): custom classifier quality 
                 indicator supported by the ClassifierExplainer object. Valid values are: 
                 'metrics', 'residuals' or'vs_col' 
@@ -784,31 +905,63 @@ class SimplifiedRegressionComposite(ExplainerComponent):
         self.index_connector = IndexConnector(self.index, [self.summary, self.contributions])
 
     def layout(self):
-        return html.Div([
-            html.H1(self.title),
-            html.H2("Model performance"),
-            dbc.CardDeck([
-                make_hideable(self.goodness_of_fit.layout(),
-                              hide=self.hide_goodness_of_fit),
-                make_hideable(self.regression_custom_component.layout(),
-                              hide=self.hide_regression_custom_component),
-            ], style=dict(marginTop=25, marginBottom=25)),
-            html.H3("SHAP values"),
-            dbc.CardDeck([
-                make_hideable(self.shap_summary.layout(),
-                              hide=self.hide_shapsummary),
-                make_hideable(self.shap_dependence.layout(),
-                              hide=self.hide_shapdependence),
-            ], style=dict(marginTop=25, marginBottom=25)),
-            html.H2("Individual predictions"),
-            dbc.CardDeck([
-                make_hideable(self.index.layout(),
-                              hide=self.hide_predindexselector),
-                make_hideable(self.summary.layout(),
-                              hide=self.hide_predictionsummary),
-            ], style=dict(marginBottom=25, marginTop=25)),
-            dbc.CardDeck([
-                make_hideable(self.contributions.layout(),
-                              hide=self.hide_contributiongraph),
-            ], style=dict(marginBottom=25, marginTop=25))
-        ])
\ No newline at end of file
+        return dbc.Container([
+            dbc.Row([
+                make_hideable(
+                    dbc.Col([
+                        html.H1(self.title, id='simple-regression-composite-title'),
+                    ]), hide=self.hide_title),
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H2("Model performance"),
+                    dbc.CardDeck([
+                        make_hideable(self.goodness_of_fit.layout(),
+                                    hide=self.hide_goodness_of_fit),
+                        make_hideable(self.regression_custom_component.layout(),
+                                    hide=self.hide_regression_custom_component),
+                    ], style=dict(marginTop=25, marginBottom=25)),
+                ])
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H3("SHAP values"),
+                    dbc.CardDeck([
+                        make_hideable(self.shap_summary.layout(),
+                                    hide=self.hide_shapsummary),
+                        make_hideable(self.shap_dependence.layout(),
+                                    hide=self.hide_shapdependence),
+                    ], style=dict(marginTop=25, marginBottom=25)),
+                ])
+            ]),
+            dbc.Row([
+                dbc.Col([
+                    html.H2("Individual predictions"),
+                    dbc.CardDeck([
+                        make_hideable(self.index.layout(),
+                                    hide=self.hide_predindexselector),
+                        make_hideable(self.summary.layout(),
+                                    hide=self.hide_predictionsummary),
+                    ], style=dict(marginBottom=25, marginTop=25)),
+                    dbc.CardDeck([
+                        make_hideable(self.contributions.layout(),
+                                    hide=self.hide_contributiongraph),
+                    ], style=dict(marginBottom=25, marginTop=25))
+                ])
+            ])   
+        ], fluid=False)
+
+    def to_html(self, state_dict=None, add_header=True):
+        html = to_html.hide(to_html.title(self.title), hide=self.hide_title)
+        html += to_html.card_rows(
+            [to_html.hide(self.goodness_of_fit.to_html(state_dict, add_header=False), hide=self.hide_goodness_of_fit),
+             to_html.hide(self.regression_custom_component.to_html(state_dict, add_header=False), hide=self.hide_regression_custom_component)],
+            [to_html.hide(self.shap_summary.to_html(state_dict, add_header=False), hide=self.hide_shapsummary),
+             to_html.hide(self.shap_dependence.to_html(state_dict, add_header=False), hide=self.hide_shapdependence)],
+            [to_html.hide(self.index.to_html(state_dict, add_header=False), hide=self.hide_predindexselector),
+             to_html.hide(self.summary.to_html(state_dict, add_header=False), hide=self.hide_predictionsummary)],
+            [to_html.hide(self.contributions.to_html(state_dict, add_header=False), hide=self.hide_contributiongraph)]
+        )
+        if add_header:
+            return to_html.add_header(html)
+        return html
\ No newline at end of file
