@@ -1,6 +1,7 @@
 __all__ = [
     'PredictionSummaryComponent',
     'ImportancesComponent',
+    'FeatureDescriptionsComponent',
     'FeatureInputComponent',
     'PdpComponent',
 ]
@@ -19,6 +20,7 @@ from dash.dependencies import Input, Output, State
 from dash.exceptions import PreventUpdate
 
 from ..dashboard_methods import *
+from .. import to_html
 
 class PredictionSummaryComponent(ExplainerComponent):
     def __init__(self, explainer, title="Prediction Summary", name=None,
@@ -108,6 +110,11 @@ class PredictionSummaryComponent(ExplainerComponent):
 
 
 class ImportancesComponent(ExplainerComponent):
+    _state_props = dict(
+        depth=('importances-depth-', 'value'),
+        importance_type=('importances-permutation-or-shap-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Feature Importances", name=None,
                         subtitle="Which features had the biggest impact?",
                         hide_type=False, hide_depth=False, hide_popout=False,
@@ -234,6 +241,17 @@ class ImportancesComponent(ExplainerComponent):
                 ], justify="end"),
             ])         
         ])
+
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        args['depth'] = None if args['depth'] is None else int(args['depth'])
+        fig =  self.explainer.plot_importances(
+                        kind=args['importance_type'], topx=args['depth'], pos_label=args['pos_label'])
+
+        html = to_html.card(to_html.fig(fig), title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
         
     def component_callbacks(self, app, **kwargs):
         @app.callback(  
@@ -249,7 +267,99 @@ class ImportancesComponent(ExplainerComponent):
             return plot
 
 
+class FeatureDescriptionsComponent(ExplainerComponent):
+
+    _state_props = dict(sort=('feature-descriptions-table-sort-', 'value'))
+
+    def __init__(self, explainer, title="Feature Descriptions", name=None, 
+                hide_title=False, hide_sort=False,
+                sort='alphabet', **kwargs):
+        """ Display Feature Descriptions table.
+        
+        Args:
+            explainer (Explainer): explainer object constructed with either
+                        ClassifierExplainer() or RegressionExplainer()
+            title (str, optional): Title of tab or page. Defaults to 
+                        "Feature Importances".
+            name (str, optional): unique name to add to Component elements. 
+                        If None then random uuid is generated to make sure 
+                        it's unique. Defaults to None.
+            hide_title (bool, optional): hide the title
+            hide_sort (bool, optional): hide the sort 
+            sort (str, optional): how to sort the features, either 'alphabet' 
+                or by mean abs shap ('shap')
+        
+        """
+        super().__init__(explainer, title, name)
+        
+        if sort not in {'alphabet', 'shap'}:
+            raise ValueError("FeatureDesriptionsComponent parameter sort should be either"
+                            "'alphabet' or 'shap'!")
+    
+    def layout(self):
+        return dbc.Card([
+            make_hideable(
+                dbc.CardHeader([
+                    html.H3(self.title), 
+                ]), hide=self.hide_title), 
+            dbc.CardBody([
+                dbc.Row([
+                    make_hideable(
+                        dbc.Col([
+                            dbc.FormGroup([
+                                dbc.Label("Sort Features:"),
+                                dbc.Select(
+                                    options=[
+                                        {'label': 'Alphabetically', 
+                                        'value': 'alphabet'},
+                                        {'label': 'SHAP', 
+                                        'value': 'shap'}
+                                    ],
+                                    value=self.sort,
+                                    id='feature-descriptions-table-sort-'+self.name,
+                                ), 
+                            ]),
+                            dbc.Tooltip("Sort features either alphabetically or from highest "
+                                        "mean absolute SHAP value to lowest.",
+                                    target='feature-descriptions-table-sort-'+self.name),
+                        ], md=3), self.hide_sort),   
+                ], form=True),
+                dbc.Row([
+                    dbc.Col([
+                        html.Div(id='feature-descriptions-table-'+self.name)
+                    ]),
+                ]),
+            ])
+        ])
+
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        html = to_html.table_from_df(self.explainer.get_descriptions_df(sort=args['sort']))
+        html = to_html.card(html, title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+    
+    def component_callbacks(self, app):
+        @app.callback(
+            Output('feature-descriptions-table-'+self.name, 'children'),
+            Input('feature-descriptions-table-sort-'+self.name, 'value')
+        )
+        def update_feature_descriptions_table(sort):
+            return dbc.Table.from_dataframe(self.explainer.get_descriptions_df(sort=sort)) 
+
+
 class PdpComponent(ExplainerComponent):
+    _state_props = dict(
+        index=('pdp-index-', 'value'),
+        col=('pdp-col-', 'value'),
+        dropna=('pdp-dropna-', 'value'),
+        sample=('pdp-sample-', 'value'),
+        gridlines=('pdp-gridlines-', 'value'),
+        gridpoints=('pdp-gridpoints-', 'value'),
+        cats_sort=('pdp-categories-sort-', 'value'),
+        pos_label=('pos-label-', 'value')
+    )
     def __init__(self, explainer, title="Partial Dependence Plot", name=None,
                     subtitle="How does the prediction change if you change one feature?",
                     hide_col=False, hide_index=False, 
@@ -442,9 +552,40 @@ class PdpComponent(ExplainerComponent):
                 ], form=True),
             ]), hide=self.hide_footer)
         ])
+
+    def get_state_tuples(self):
+        _state_tuples = super().get_state_tuples()
+        if self.feature_input_component is not None:
+            _state_tuples.extend(self.feature_input_component.get_state_tuples())
+        return sorted(list(set(_state_tuples)))
+
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        if self.feature_input_component is None:
+            fig = self.explainer.plot_pdp(args['col'], args['index'], 
+                        drop_na=bool(args['dropna']), sample=args['sample'], 
+                        gridlines=args['gridlines'], gridpoints=args['gridpoints'], 
+                        sort=args['cats_sort'], pos_label=args['pos_label'])
+            html = to_html.fig(fig)
+        else:
+            inputs = {k:v for k,v in self.feature_input_component.get_state_args(state_dict).items() if k != 'index'}
+            inputs = list(inputs.values())
+            if len(inputs) == len(self.feature_input_component._input_features) and not any([i is None for i in inputs]):
+                X_row = self.explainer.get_row_from_input(inputs, ranked_by_shap=True)
+                fig = self.explainer.plot_pdp(args['col'], X_row=X_row, 
+                        drop_na=bool(args['dropna']), sample=args['sample'], 
+                        gridlines=args['gridlines'], gridpoints=args['gridpoints'], 
+                        sort=args['cats_sort'], pos_label=args['pos_label'])
+                html = to_html.fig(fig)
+            else:
+                html = f"<div>input data incorrect<div>"
+
+        html = to_html.card(html, title=self.title)
+        if add_header:
+            return to_html.add_header(html)
+        return html
                 
     def component_callbacks(self, app):
-
         @app.callback(
             Output('pdp-categories-sort-div-'+self.name, 'style'),
             Input('pdp-col-'+self.name, 'value')
@@ -551,6 +692,10 @@ class FeatureInputComponent(ExplainerComponent):
                 feature, self.explainer.onehot_cols, self.explainer.onehot_dict, self.explainer.categorical_dict) 
                                 for feature in self._input_features]
 
+        self._state_props = {feature: (f'feature-input-{feature}-input-', 'value') 
+                                for feature in self._input_features}
+        self._state_props['index'] = ('feature-input-index-', 'value')
+
         if self.description is None: self.description = """
         Adjust the input values to see predictions for what if scenarios."""
 
@@ -646,6 +791,20 @@ class FeatureInputComponent(ExplainerComponent):
             ])
         ])
 
+    def to_html(self, state_dict=None, add_header=True):
+        args = self.get_state_args(state_dict)
+        html_inputs = [to_html.input(feature, args.get(feature, None), disabled=True) for feature in self._input_features]
+        html = to_html.hide(f"Selected: <b>{args['index']}</b>", hide=self.hide_index)
+        if self.fill_row_first:
+            html += to_html.row(*["".join(html_inputs[slicer]) for slicer in self.get_slices_rows_first(len(self._input_features), self.n_input_cols)])
+        else:
+            html += to_html.row(*["".join(html_inputs[slicer]) for slicer in self.get_slices_cols_first(len(self._input_features), self.n_input_cols)])
+        
+        html = to_html.card(html, title=self.title, subtitle=self.subtitle)
+        if add_header:
+            return to_html.add_header(html)
+        return html
+
     def component_callbacks(self, app):
         @app.callback(
             [*self._feature_callback_outputs],
