@@ -1,3 +1,35 @@
+__all__ = [
+    'IndexNotFoundError',
+    'safe_isinstance',
+    'guess_shap',
+    'mape_score',
+    'parse_cats',
+    'get_encoded_and_regular_cols',
+    'split_pipeline',
+    'retrieve_onehot_value',
+    'merge_categorical_columns',
+    'matching_cols',
+    'remove_cat_names',
+    'X_cats_to_X',
+    'merge_categorical_shap_values',
+    'merge_categorical_shap_interaction_values',
+    'make_one_vs_all_scorer',
+    'permutation_importances',
+    'cv_permutation_importances',
+    'get_mean_absolute_shap_df',
+    'get_grid_points',
+    'get_pdp_df',
+    'get_precision_df',
+    'get_liftcurve_df',
+    'get_contrib_df',
+    'get_contrib_summary_df',
+    'normalize_shap_interaction_values',
+    'get_decisionpath_df',
+    'get_decisiontree_summary_df',
+    'get_xgboost_node_dict',
+    'get_xgboost_path_df',
+    'get_xgboost_path_summary_df',
+    'get_xgboost_preds_df']
 
 from functools import partial
 import re
@@ -66,6 +98,10 @@ def guess_shap(model):
                   ]
     linear_models = ['LinearRegression', 'LogisticRegression',
                     'Ridge', 'Lasso', 'ElasticNet', 'SGDClassifier']
+
+    skorch_models = ['skorch.net.NeuralNet',
+                     'skorch.regressor.NeuralNetRegressor',
+                     'skorch.classifier.NeuralNetClassifier']
     
     for tree_model in tree_models:
         if str(type(model)).endswith(tree_model + "'>"):
@@ -74,7 +110,11 @@ def guess_shap(model):
     for lin_model in linear_models:
         if str(type(model)).endswith(lin_model + "'>"):
             return 'linear'
-    
+
+    for skorch_model in skorch_models:
+        if str(type(model)).endswith(skorch_model + "'>"):
+            return 'skorch'
+
     return None
 
 
@@ -428,7 +468,8 @@ def make_one_vs_all_scorer(metric, pos_label=1, greater_is_better=True):
 
 def permutation_importances(model, X, y, metric, onehot_dict=None,
                             greater_is_better=True, needs_proba=False,
-                            pos_label=1, n_repeats=1, n_jobs=None, sort=True, verbose=0):
+                            pos_label=1, n_repeats=1, n_jobs=None, sort=True, 
+                            pass_nparray=False, verbose=0):
     """
     adapted from rfpimp package, returns permutation importances, optionally grouping 
     onehot-encoded features together.
@@ -451,6 +492,8 @@ def permutation_importances(model, X, y, metric, onehot_dict=None,
             Defaults to 1.
         n_jobs (int): number of jobs for joblib parallel. Defaults to None. 
         sort (bool): sort the output from highest importances to lowest. 
+        pass_nparray (bool, optional): instead of the X pass X.values to model.
+            This is useful for skorch models that do not accepts dataframes.
         verbose (int): set to 1 to print output for debugging. Defaults to 0.
     """
     X = X.copy()
@@ -464,22 +507,30 @@ def permutation_importances(model, X, y, metric, onehot_dict=None,
         scorer = make_scorer(metric, greater_is_better=greater_is_better, needs_proba=needs_proba)
     else:
         scorer = make_one_vs_all_scorer(metric, pos_label, greater_is_better)
+    if pass_nparray:
+        baseline = scorer(model, X.values, y.values)
+    else:
+        baseline = scorer(model, X, y)
 
-    baseline = scorer(model, X, y)
 
-    def _permutation_importance(model, X, y, scorer, col_name, col_list, baseline, n_repeats=1):
+    def _permutation_importance(model, X, y, scorer, col_name, col_list, 
+                baseline, n_repeats=1, pass_nparray=False):
         X = X.copy()
         scores = []
         for i in range(n_repeats):
             old_cols = X[col_list].copy()
             X[col_list] = np.random.permutation(X[col_list])
-            scores.append(scorer(model, X, y))
+            if pass_nparray:
+                scores.append(scorer(model, X.values, y.values))
+            else:
+                scores.append(scorer(model, X, y))
+
             X[col_list] = old_cols
         return col_name, np.mean(scores)
     
     scores = Parallel(n_jobs=n_jobs)(delayed(_permutation_importance)(
-                    model, X, y, scorer, col_name, col_list, baseline, n_repeats
-            ) for col_name, col_list in onehot_dict.items())
+                    model, X, y, scorer, col_name, col_list, 
+                    baseline, n_repeats, pass_nparray) for col_name, col_list in onehot_dict.items())
     
     importances_df = pd.DataFrame(scores, columns=['Feature', 'Score'])
     importances_df['Importance'] = baseline - importances_df['Score']
@@ -492,7 +543,7 @@ def permutation_importances(model, X, y, metric, onehot_dict=None,
 
 def cv_permutation_importances(model, X, y, metric, onehot_dict=None, greater_is_better=True,
                                 needs_proba=False, pos_label=None, cv=None, 
-                                n_repeats=1, n_jobs=None, verbose=0):
+                                n_repeats=1, n_jobs=None, pass_nparray=False, verbose=0):
     """
     Returns the permutation importances averages over `cv` cross-validated folds.
 
@@ -512,6 +563,8 @@ def cv_permutation_importances(model, X, y, metric, onehot_dict=None, greater_is
             Defaults to 1.
         cv (int): number of cross-validation folds to apply.
         sort (bool): sort the output from highest importances to lowest. 
+        pass_nparray (bool, optional): instead of the X pass X.values to model.
+            This is useful for skorch models that do not accepts dataframes.
         verbose (int): set to 1 to print output for debugging. Defaults to 0.
     """
     if cv is None:
@@ -522,6 +575,7 @@ def cv_permutation_importances(model, X, y, metric, onehot_dict=None, greater_is
                                         n_repeats=n_repeats,
                                         n_jobs=n_jobs,
                                         sort=False,
+                                        pass_nparray=pass_nparray,
                                         verbose=verbose)
 
     if needs_proba:
@@ -545,6 +599,7 @@ def cv_permutation_importances(model, X, y, metric, onehot_dict=None, greater_is
                                         n_repeats=n_repeats,
                                         n_jobs=n_jobs,
                                         sort=False,
+                                        pass_nparray=pass_nparray,
                                         verbose=verbose)
         if i == 0:
             imps = imp[['Feature', 'Importance']]
@@ -619,7 +674,8 @@ def get_grid_points(array, n_grid_points=10, min_percentage=0, max_percentage=10
 
 def get_pdp_df(model, X_sample:pd.DataFrame, feature:Union[str, List], pos_label=1,
                   n_grid_points:int=10, min_percentage:int=0, max_percentage:int=100,
-                  multiclass:bool=False, grid_values:List=None):
+                  multiclass:bool=False, grid_values:List=None, is_classifier:bool=False,
+                  cast_to_float32:bool=False):
     """Returns a dataframe with partial dependence for every row in X_sample for a number of feature values
 
     Args:
@@ -641,6 +697,9 @@ def get_pdp_df(model, X_sample:pd.DataFrame, feature:Union[str, List], pos_label
             one for each predicted label.
         grid_values (list, optional): list of grid values. Default to None, in which
             case it will be inferred from X_sample.
+        is_classifier (bool, optional): model is a classifier with a pred_probas method.
+        cast_to_float32 (bool, optional): cast model input to np.float32 (necessary for
+            skorch models)
     """
     
 
@@ -659,8 +718,12 @@ def get_pdp_df(model, X_sample:pd.DataFrame, feature:Union[str, List], pos_label
             raise ValueError("feature should either be a column name (str), "
                              "or a list of onehot-encoded columns!")
 
-    if hasattr(model, "predict_proba"):
-        n_labels = model.predict_proba(X_sample.iloc[[0]]).shape[1]
+    if is_classifier:
+        if cast_to_float32:
+            first_row = X_sample.iloc[[0]].values.astype("float32")
+        else:
+            first_row = X_sample.iloc[[0]]
+        n_labels = model.predict_proba(first_row).shape[1]
         if multiclass:
             pdp_dfs = [pd.DataFrame() for i in range(n_labels)]
         else:
@@ -677,15 +740,19 @@ def get_pdp_df(model, X_sample:pd.DataFrame, feature:Union[str, List], pos_label
             dtemp.loc[:, feature] = [1 if col==grid_value else 0 for col in feature]
         else:
             dtemp.loc[:, feature] = grid_value
-        if hasattr(model, "predict_proba"):
-            pred_probas = model.predict_proba(dtemp)
+        if is_classifier:
+            if cast_to_float32:
+                dtemp = dtemp.values.astype("float32")
+            pred_probas = model.predict_proba(dtemp).squeeze()
             if multiclass:
                 for i in range(n_labels):
                     pdp_dfs[i][grid_value] = pred_probas[:, i]
             else:
                 pdp_df[grid_value] = pred_probas[:, pos_label]
         else:
-            preds = model.predict(dtemp)  
+            if cast_to_float32:
+                dtemp = dtemp.values.astype("float32")
+            preds = model.predict(dtemp).squeeze()
             pdp_df[grid_value] = preds
     if multiclass:
         return pdp_dfs
@@ -1346,17 +1413,17 @@ def get_xgboost_preds_df(xgbmodel, X_row, pos_label=1):
     if is_classifier:
         if n_classes == 2:
             if pos_label==1:
-                preds = [xgbmodel.predict(X_row, ntree_limit=i+1, output_margin=True)[0] for i in range(n_trees)]
+                preds = [xgbmodel.predict(X_row, iteration_range=(0, i+1), output_margin=True)[0] for i in range(n_trees)]
             elif pos_label==0:
-                preds = [-xgbmodel.predict(X_row, ntree_limit=i+1, output_margin=True)[0] for i in range(n_trees)]
+                preds = [-xgbmodel.predict(X_row, iteration_range=(0, i+1), output_margin=True)[0] for i in range(n_trees)]
             pred_probas = (np.exp(preds)/(1+np.exp(preds))).tolist()
         else:
-            margins = [xgbmodel.predict(X_row, ntree_limit=i+1, output_margin=True)[0] for i in range(n_trees)]
+            margins = [xgbmodel.predict(X_row, iteration_range=(0, i+1), output_margin=True)[0] for i in range(n_trees)]
             preds = [margin[pos_label] for margin in margins]
             pred_probas = [(np.exp(margin)/ np.exp(margin).sum())[pos_label] for margin in margins]
             
     else:
-        preds = [xgbmodel.predict(X_row, ntree_limit=i+1, output_margin=True)[0] for i in range(n_trees)]
+        preds = [xgbmodel.predict(X_row, iteration_range=(0, i+1), output_margin=True)[0] for i in range(n_trees)]
              
     
     xgboost_preds_df = pd.DataFrame(
